﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Controls.Primitives;
using System.Windows.Input;

namespace uVisStudio.VisPropertyBox
{
    public partial class PropertyBox : UserControl, INotifyPropertyChanged
    {
        public event PropertyChangedEventHandler PropertyChanged;

        private int gridSplitterPosition1 = 1;
        private int gridSplitterPosition2 = 1;

        public int GridSplitterPosition1
        {
            get { return gridSplitterPosition1; }
            set 
            { 
                this.gridSplitterPosition1 = value;
                if(PropertyChanged != null)
                    PropertyChanged.Invoke(this, new PropertyChangedEventArgs("GridSplitterPosition1"));
            }
        }

        public int GridSplitterPosition2
        {
            get { return gridSplitterPosition2; }
            set 
            { 
                this.gridSplitterPosition2 = value;
                if(PropertyChanged != null)
                    PropertyChanged.Invoke(this, new PropertyChangedEventArgs("GridSplitterPosition2"));
            }
        }

        public PropertyBox()
        {
            InitializeComponent();
            this.propertiesGrid.SizeChanged += this.PropertiesGridSizeChanged;
            this.DescriptionGrid.DataContext = this;
            this.DescriptionGrid.Background = ColorConstants.Normal();
        }

        public void Update(List<UserControl> controlItems)
        {
            ControlItems.Children.Clear();
            ControlItems.Background = ColorConstants.Normal();
            foreach (var itemControl in controlItems)
            {
                ControlItems.DataContext = this;
                ControlItems.Children.Add(itemControl);
            }
        }

        private void PropertiesGridSizeChanged(object sender, SizeChangedEventArgs e)
        {
            Resizing();
        }

        private void GridSplitterDragDelta(object sender, DragDeltaEventArgs dragDeltaEventArgs)
        {
            Resizing();
        }

        private void Resizing()
        {
            var value1 = propertiesGrid.ColumnDefinitions[1].ActualWidth + 2;
            var value2 = propertiesGrid.ColumnDefinitions[3].ActualWidth +2;
            this.GridSplitterPosition1 = (int)value1;
            if (value1 < 0)
            {
                this.GridSplitterPosition1 = 0;
            }
            this.GridSplitterPosition2 = (int)value2;
            if (value2 < 0)
            {
                this.GridSplitterPosition2 = 0;
            }
        }
    }
}
