﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Drawing;
using System.Reflection;
using System.Windows;
using System.Linq;
using System.Windows.Controls;
using System.Windows.Controls.Primitives;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using VisTool;
using uVisStudio.VisDesigner;
using uVisStudio.VisExplorer;
using Button = System.Windows.Controls.Button;
using Image = System.Windows.Controls.Image;

namespace uVisStudio.VisPropertyBox {
    public partial class PropertyBoxMenu : UserControl {
        private Controller controller = Controller.Instance;
        private ContextMenu popup = new ContextMenu();
        private bool userChanged;
        private bool formUserChanged;
        //public static event EventHandler SelectionChanged;

        public PropertyBoxTemplate PropertyGrid;
        public Template CurrentTemplate;
        public VisComponent VisComponent;

        public RadioButton Categorized {
            get { return this.categorized; }
            set { this.categorized = value; }
        }

        public RadioButton Alphabetical {
            get { return this.alphabetical; }
            set { this.alphabetical = value; }
        }

        public Button NewPropertyButton {
            get { return this.addButton; }
        }

        public Button RemovePropertyButton {
            get { return this.removeButton; }
        }

        public PropertyBoxMenu() {
            this.InitializeComponent();
            this.Background = ColorConstants.MenuBackGround();
            this.categoriesImage.Source = Properties.Resources.Properties.GetImage();
            this.abcImage.Source = Properties.Resources.SortHS.GetImage();

            this.addImage.Source = Properties.Resources.Add.GetImage();
            this.removeImage.Source = Properties.Resources.Remove.GetImage();
            this.selectComponentImage.Source = Properties.Resources.selectImage.GetImage();
            this.deleteComponentImage.Source = Properties.Resources.deleteImage.GetImage();
            //this.undoButtonImage.Source = Properties.Resources.Edit_UndoHS.GetImage();
            //this.redoButtonImage.Source = Properties.Resources.Edit_RedoHS.GetImage();
            this.formComboBox.SelectionChanged += this.FormComboBoxSelectionChanged;
            this.templateComboBox.SelectionChanged += ComboBoxTemplateSelectionChanged;
            this.templateComboBox.DropDownOpened += ComboBoxTemplateDropDownOpened;
            this.formComboBox.DropDownOpened += this.FormComboBoxTemplateDropDownOpened;

            //if (!Menus.Contains(this))
            // Menus.Add(this);
            //StudioBase.Studio.propertyBoxManager.StudioGrids[0].Grid.m
        }

        public void SetupEvents() {
            this.Categorized.Checked += PropertyGrid.Update;
            this.Alphabetical.Checked += PropertyGrid.Update;
            this.deleteComponent.Click += PropertyGrid.deleteThisComponent;
            this.highlightComponent.Click += PropertyGrid.selectThisComponent;
            //this.redoButton.Click += CallRedoCommand;
            //this.undoButton.Click += CallUndoCommand;
        }

        public void CallRedoCommand(object sender, RoutedEventArgs e) {
            UndoManager.Redo();
        }

        public void CallUndoCommand(object sender, RoutedEventArgs e) {
            UndoManager.Undo();
        }

        private void makeComboBoxesConsistent() {
            this.userChanged = false;
            this.formUserChanged = false;
            FormTemplate formTemplate = CurrentTemplate.FormTpl;
            KeyValuePair<string, FormTemplate> curFormComboboxValue = new KeyValuePair<string, FormTemplate>(formTemplate.Name, formTemplate);
            KeyValuePair<string, Template> curComboboxValue = new KeyValuePair<string, Template>(this.CurrentTemplate.Name, this.CurrentTemplate);
            formComboBox.SelectedItem = curFormComboboxValue;
            templateComboBox.SelectedItem = curComboboxValue;
        }

        public static StudioGrid FindStudioGrid(PropertyBoxMenu menu) {
            foreach (StudioGrid sGrid in PropertyBoxManager.StudioGrids)
                if (menu == sGrid.Grid.menu)
                    return sGrid;
            return null;
        }

        private void highlightSelectedTemplate(Template template) {
            SelectionBox box;
            StudioGrid grid = FindStudioGrid(this);
            VisComponent currentComp = null;
            if (SelectionBox.selections.TryGetValue(template, out box)) // One of the components has been selected. Show it.
                currentComp = box.cmp;
            else if (template.RowBundle != null && template.RowBundle.Instances.Count > 0) { // No selected component. Show the first in the first bundle.
                Bundle b = template.RowBundle;
                if (b.Instances.Count > 0) currentComp = b.Instances[0].cmp;
            }
            if (currentComp != null)
                TemplateHandler.SelectTemplate(currentComp, grid);
            else
                TemplateHandler.SelectTemplate(template, grid);
        }

        private void ComboBoxTemplateDropDownOpened(object sender, EventArgs e) {
            this.userChanged = true;
            ComboBox cb = sender as ComboBox;
            //foreach (var item in cb.ItemsSource) { // Is a list of (name, Template)
            //    Debug.Print(item.ToString()); break;
            //}
            //foreach (var item in cb.Items) { // Is the same list of (name, Template)
            //    Debug.Print(item.ToString()); break;
            //}
        }

        private void ComboBoxTemplateTypeDropDownOpened(object sender, EventArgs e) {
            this.userChanged = true;
        }

        private void FormComboBoxTemplateDropDownOpened(object sender, EventArgs e) {
            this.formUserChanged = true;
        }

        private void FormComboBoxSelectionChanged(object sender, SelectionChangedEventArgs e) {
            if (!formUserChanged) return;
            FormTemplate formTemplate = this.formComboBox.SelectedValue as FormTemplate; ;
            if (formTemplate == null)
                return;
            //PopulateOnlyTemplateCombobox(formTemplate);
            highlightSelectedTemplate(formTemplate);
            makeComboBoxesConsistent();
        }

        private void ComboBoxTemplateSelectionChanged(object sender, SelectionChangedEventArgs e) {
            if (!userChanged)
                return;
            if (this.templateComboBox.SelectedValue != null) {
                Template template = (Template)this.templateComboBox.SelectedValue;
                highlightSelectedTemplate(template);
                makeComboBoxesConsistent();
            }
        }

        public void SelectTemplate(Template template) {
            this.PopulateTemplateCombobox(template.FormTpl);
            var curComboboxValue = new KeyValuePair<string, Template>(template.Name + (template.Ignore ? "    Ignore" : ""), template);
            this.templateComboBox.SelectedItem = curComboboxValue;
        }

        public void SelectTemplateOnly(Template template) {
            this.PopulateOnlyTemplateCombobox(template);
            var curComboboxValue = new KeyValuePair<string, Template>(template.Name, template);
            this.templateComboBox.SelectedItem = curComboboxValue;
        }

        public void Reset() {
            this.CurrentTemplate = null;
            this.VisComponent = null;
            //this.templateComboBox.ItemsSource = null;
            this.templateComboBox.SelectedItem = null;

            //ResetFormComboBoxes();

            //make sure all form comboboxes are updated
            foreach (StudioGrid sGrid in PropertyBoxManager.StudioGrids)
                sGrid.Grid.menu.ResetFormComboBoxes();
            //this.formComboBox.ItemsSource = null;
            //this.formComboBox.SelectedItem = null;
        }

        public void ResetFormComboBoxes() {
            formUserChanged = false;
            //formComboBox.ItemsSource = null;
            FormTemplate selectedFrm = formComboBox.SelectedItem != null ? formComboBox.SelectedItem as FormTemplate : null;
            PopulateFormCombobox(selectedFrm);

            //KeyValuePair<string, FormTemplate> menuSelectedForm = formComboBox.SelectedItem != null ? (KeyValuePair<string, FormTemplate>)formComboBox.SelectedItem : new KeyValuePair<string, FormTemplate>();
            //formComboBox.ItemsSource = null;
            //formComboBox.ItemsSource = VisEngine.Forms;
            //formComboBox.DisplayMemberPath = "Key";
            //formComboBox.SelectedValuePath = "Value";
            //    if (menuSelectedForm.Key != null && VisEngine.Forms.ContainsKey(menuSelectedForm.Key))
            //        formComboBox.SelectedItem = menuSelectedForm;
        }

        private void PopulateFormCombobox(FormTemplate formTemplate) { // formTemplate: The form to show as selected
            //make sure all form comboboxes are updated
            this.formUserChanged = false;
            this.formComboBox.ItemsSource = null;

            Dictionary<string, FormTemplate> openForms = new Dictionary<string, FormTemplate>(); // Each form may exist only once.
            foreach (var formItem in VisEngine.Forms) {
                List<FormTemplate> clones = formItem.Value;
                if (clones != null) {
                    for (int i = 0; i < clones.Count; i++)
                        if (clones[i] != null && clones[i].open) { openForms.Add(formItem.Key, clones[i]); break; }
                }
            }
            //if (openForms.Count == 0) return;

            var orderedForms = from v in openForms orderby v.Key ascending select v;
            this.formComboBox.ItemsSource = orderedForms;
            this.formComboBox.DisplayMemberPath = "Key";
            this.formComboBox.SelectedValuePath = "Value";
            KeyValuePair<string, FormTemplate> curFormComboboxValue = new KeyValuePair<string, FormTemplate>(formTemplate.Name, formTemplate);
            this.formComboBox.SelectedItem = curFormComboboxValue;
            //Trigger tr = new Trigger();
            //this.formComboBox.Triggers.Add(new Trigger()) ;
        }

        private void PopulateOnlyTemplateCombobox(Template template) {
            var formTemplate = template.FormTpl;
            this.userChanged = false;
            Dictionary<string, Template> temp = new Dictionary<string,VisTool.Template>(50);
            foreach (KeyValuePair<string, Template> kvp in formTemplate.Templates)
                temp.Add(kvp.Key + (kvp.Value.Ignore ? "    Ignore" : ""), kvp.Value);
            var orderedTpls = from v in temp orderby v.Key ascending select v;
            this.templateComboBox.ItemsSource = orderedTpls;
            this.templateComboBox.DisplayMemberPath = "Key";
            this.templateComboBox.SelectedValuePath = "Value";

            var curComboboxValue = new KeyValuePair<string, Template>(template.Name, template);
            this.templateComboBox.SelectedItem = curComboboxValue;
        }

        private void PopulateTemplateCombobox(Template template) {
            var formTemplate = template.FormTpl;
            PopulateFormCombobox(formTemplate);
            PopulateOnlyTemplateCombobox(template);
        }


        private void PopupOpening(object sender, ContextMenuEventArgs e) {
            e.Handled = true;
        }

        public class CategoryData {
            public string Name;
            public string Icon;
            public List<PropInfo> Properties = new List<PropInfo>();
            public CategoryData(string name, string icon) {
                this.Name = name;
                this.Icon = icon;
            }
        }

        public static List<CategoryData> CreateCategories(VisTool.ComponentInfo cmpInfo) { // Another CreateCategories in PropertyBoxTemplate.
            // Create a list of categories with properties sorted according to Position and Name.
            var categories = new List<CategoryData>(10) // Insert the known categories in a designer-friendly sequence. SL 22-04-13
            { new CategoryData("Common", "field.png"), new CategoryData("Special", "special.png"), new CategoryData("EventHandler", "eventhandler.gif") };

            foreach (var p in cmpInfo.PropertyByName) { // Use value and event properties that are Exposed.
                if (p.Value.VisAttribute.Exposed && p.Value.VisAttribute.PropType != VisPropAtt.PropertyType.Method) {
                    string catName = p.Value.VisAttribute.PropertyCategory;
                    CategoryData c = categories.Find(delegate(CategoryData i) { return i.Name == catName; });
                    if (c == null) {
                        c = new CategoryData(catName, "");
                        categories.Add(c);
                    }
                    c.Properties.Add(p.Value);
                }
            }
            foreach (var c in categories) { // Sort properties on Position, Name.
                c.Properties.Sort((a, b) => a.VisAttribute.Position.CompareTo(b.VisAttribute.Position) != 0 ?
                    a.VisAttribute.Position.CompareTo(b.VisAttribute.Position) : a.Name.CompareTo(b.Name));
            }
            return categories;
        }

        private void NewPropertyButtonClicked(object sender, RoutedEventArgs e) {
            popup.PlacementTarget = this.addButton;
            popup.Placement = PlacementMode.Bottom;
            popup.IsOpen = true;
            popup.Items.Clear();
            if (CurrentTemplate == null) return;

            var designIcon = new Image(); // Show designer first
            designIcon.Source = new BitmapImage(new Uri(VisEngine.ProgramDirectory + "\\Icons\\designer.png"));
            designIcon.Height = 13;
            designIcon.Width = 13;
            var designer = new MenuItem();
            designer.Header = "Designer...";
            designer.Icon = designIcon;
            designer.ToolTip = "Add a custom property";
            designer.Click += MenuItemClick;
            popup.Items.Add(designer);

            var categories = CreateCategories(CurrentTemplate.Info);

            foreach (var c in categories) {
                foreach (var propInfo in c.Properties) {
                    if (propInfo.Name == "Name") continue; // Don't show the name property.
                    //var fieldPath = new BitmapImage(new Uri(VisEngine.ProgramDirectory + "\\Icons\\field.png"));
                    //var readFieldPath = new BitmapImage(new Uri(VisEngine.ProgramDirectory + "\\Icons\\field_readonly.png"));
                    //var functionPath = new BitmapImage(new Uri(VisEngine.ProgramDirectory + "\\Icons\\function.gif"));
                    //var readFunctionPath = new BitmapImage(new Uri(VisEngine.ProgramDirectory + "\\Icons\\function_readonly.gif"));
                    //var eventPath = new BitmapImage(new Uri(VisEngine.ProgramDirectory + "\\Icons\\eventhandler.gif"));
                    //var readEventPath = new BitmapImage(new Uri(VisEngine.ProgramDirectory + "\\Icons\\eventhandler_readonly.png"));
                    //var unknownPath = new BitmapImage(new Uri(VisEngine.ProgramDirectory + "\\Icons\\unknown.png"));
                    //var readUnknownPath = new BitmapImage(new Uri(VisEngine.ProgramDirectory + "\\Icons\\unknown_readonly.png"));
                    //var icon = new Image();
                    //icon.Width = 16;
                    //icon.Height = 16;
                    //icon.Source = new BitmapImage(new Uri(VisEngine.ProgramDirectory + "\\Icons\\"+c.Icon));

                    //if (attribute.PropType == VisPropAtt.PropertyType.Property) icon.Source = fieldPath;
                    //if (!attribute.ReadOnly && attribute.PropType == VisPropAtt.PropertyType.SystemMethod) icon.Source = functionPath;
                    //if (!attribute.ReadOnly && attribute.PropType == VisPropAtt.PropertyType.Eventhandler) icon.Source = eventPath;
                    //if (!attribute.ReadOnly && attribute.PropType == VisPropAtt.PropertyType.Unknown) icon.Source = unknownPath;
                    //if (attribute.ReadOnly && attribute.PropType == VisPropAtt.PropertyType.Property) icon.Source = readFieldPath;
                    //if (attribute.ReadOnly && attribute.PropType == VisPropAtt.PropertyType.SystemMethod) icon.Source = readFunctionPath;
                    //if (attribute.ReadOnly && attribute.PropType == VisPropAtt.PropertyType.Eventhandler) icon.Source = readEventPath;
                    //if (attribute.ReadOnly && attribute.PropType == VisPropAtt.PropertyType.Unknown) icon.Source = readUnknownPath;

                    var attribute = propInfo.VisAttribute;
                    var item = new MenuItem();
                    item.Header = propInfo.Name;
                    var icon = new Image();
                    icon.Width = 16;
                    icon.Height = 16;
                    icon.Source = new BitmapImage(new Uri(VisEngine.ProgramDirectory + "\\Icons\\" + c.Icon));
                    item.Icon = icon;
                    item.ToolTip = attribute.LongHelp == null ? attribute.ShortHelp : attribute.LongHelp;
                    item.Tag = propInfo;
                    item.Click += MenuItemClick;
                    popup.Items.Add(item);
                }
            }
        }

        private void MenuItemClick(object sender, RoutedEventArgs e) {
            if (CurrentTemplate == null) return;
            var item = (MenuItem)sender;
            var propInfo = item.Tag as PropInfo;
            this.controller.AddTemplateProperty(CurrentTemplate, propInfo); // Adds only if present formula is null.
            // Set focus to the grid line. Clumsy algorithm - how to make it shorter? SL 17-05-15.
            foreach (var studioGrid in PropertyBoxManager.StudioGrids) {
                foreach (Category c in studioGrid.Grid.Categories)
                    foreach (PropertyGridLine prop in c.Properties) {
                        TplProperty tplProp = prop.Property.TemplateProperty;
                        if (propInfo == null) {
                            if (tplProp is TplPropDesigner) {
                                prop.formulaEditor.Focus();
                                return;
                            }
                        } else if (tplProp.tpl == CurrentTemplate && tplProp.cmpPropIndex == propInfo.propIndex
                            && (tplProp is TplPropValue && propInfo is ValueInfo || tplProp is TplPropEvent && propInfo is EvtInfo)) {
                            prop.formulaEditor.Focus();
                            return;
                        }
                    }
            }
        }

        private void RemovePropertyButtonClicked(object sender, RoutedEventArgs e) {
            var template = CurrentTemplate;
            if (PropertyGrid.SelectedProperty != null) {
                var property = PropertyGrid.SelectedProperty.Property;
                //if (property.TemplateProperty.PropKind == VisTool.ProprtyKind.GUI || property.TemplateProperty.PropKind == VisTool.ProprtyKind.Standard)
                //    MessageBox.Show("This property can not be deleted!");
                //else
                if (property != null && property.TemplateProperty.Name != null)
                    this.controller.RemoveTemplateProperty(template, property.TemplateProperty.Name);
            } else MessageBox.Show("Select a property to delete", "uVis Studio");

        }
    }
}